--[[
   CrossDevExample.lua - Cross development script that compute fx tables,
                         compile the fx (from assembly code) and run it

   $VER: CrossDevExample.lua 1.2 (23.02.17)  2015-2017 by Philippe 'OffseT' Rimauro <offset@cpcscene.net>
   Requires ACE 1.14 or later
            vasm targeting Z80 to be in the path (http://sun.hasenbraten.de/vasm)
]]

require 'ipc'
require 'io'
require 'math'

--[[ Degree to radian ]]
function Deg2Rad(angle)
    return angle/180*math.pi
end

aceport='ACE.1'

--[[ Look for ACE running ]]
if not ipc.checkport('ACE.1') then
    address('command');
    rx('RequestChoice >NIL: TITLE "CrossDevExample.lua" BODY "ACE CPC Emulator process not found!" GADGETS "Oops!"')
    return
end

--[[ Main ]]
address(aceport)
io.write('ACE version '..rx('VERSION')..' found.\n\n')

--[[ Reset ACE ]]
io.write('Cleaning up everything and preparing the CPC...\n')
rx('RESET HARD')
--[[ Let the firmware boot ]]
address('command') rx('Wait',1)
-- [[ Prepare the CPC ]]
address(aceport)
rx('KEYBOARDSTROKE WAIT SYMBOL AFTER 256')
rx('KEYBOARDSTROKE WAIT MEMORY &7FFF')
io.write('Done.\n\n')

--[[ Pause ACE while working ]]
rx('PAUSE')

--[[ Compute tables ]]
io.write('Computing & poking tables into ACE...\n')
local a=0x9000
local b=-0.2
local n=52
local x=0
local val
for ang=0,359,2 do
    n=n+b
    val=math.floor((math.sin(Deg2Rad(ang))*math.cos(Deg2Rad(ang))+1)*n)
    rx('POKE',a,val)
    --io.write(x..' -> '..val..'\n')
    if ang==180 then b=-b end
    a=a+1
    x=x+1
end
io.write('1st table done: '..x..' value computed!\n')

a=0x9800
x=0
for ang=-90,629,6 do
    val=math.floor((math.sin(Deg2Rad(ang))+1)*23+46)
    rx('POKE',a,val)
    --io.write(x..' -> '..val..'\')
    a=a+1
    x=x+1
end
for ang=90,809,6 do
    val=math.floor((math.sin(Deg2Rad(ang))+1)*23)
    rx('POKE',a,val)
    --io.write(x..' -> '..val..'\n')
    a=a+1
    x=x+1
end
io.write('2nd table done: '..x..' value computed!\n')

--[[ Assemble code ]]
io.write('Assembling source code...')
address('command')
rx('vasmz80_oldstyle -Fbin -nocase -quiet -o T:CrossDevExample.bin CrossDevExample.asm')
--[[ Check for assembling error... ]]
address(aceport)
if not rc then
    rx('START')
    return
end
io.write('Done.\n\n')

--[[ Load code into ACE! ]]
io.write('Loading code into ACE...\n')
rx('LOAD FILE T:CrossDevExample.bin ADR 0x8000')
--address('command')
--rx('Delete T:CrossDevExample.bin QUIET')
--address(aceport)
io.write('Done.\n\n')

--[[ Start ACE again ]]
rx('START')
--[[ Auto-launch the assembled program ]]
io.write('Auto-launching the assembled program!\n')
rx('KEYBOARDSTROKE CALL &8000')
io.write('Done.\n\n')

return

