/* CrossDevExample.rexx - Cross development script that compute fx tables,
 *                        compile the fx (from assembly code) and run it
 *
 * $VER: CrossDevExample.rexx 1.5 (24.12.17)  2015-2017 by Philippe 'OffseT' Rimauro <offset@cpcscene.net>
 * Requires ACE 1.14 or later
 *          rexxmathlib.library (http://aminet.net/package/util/rexx/RexxMathLib)
 *          vasm targeting Z80 to be in the path (http://sun.hasenbraten.de/vasm)
 */

OPTIONS RESULTS

SIGNAL ON BREAK_C

ace_required_version="1.14"
ace_port="ACE.1"

appname="ACE Rexx"
LF="0A"x

/* Look for ACE running */
IF ~SHOW("P",ace_port) THEN DO
    EXIT FatalError("ACE CPC Emulator process not found!","Oops!")
END

/* ACE is not the default port */
ADDRESS(ace_port)

/* Check ACE version */
"VERSION"
IF CheckVersion(RESULT,ace_required_version)~=0 THEN DO
    EXIT FatalError("ACE CPC Emulator version" ace_required_version "or better is required!","I will update!")
END

/* Load required external libraries */
LoadLib("rexxsupport.library")
LoadLib("rexxmathlib.library")

/* Reset ACE */
SAY "Cleaning up everything and preparing the CPC..."
"RESET HARD"

/* Let the firmware boot and initialize the CPC memory */
DELAY(50)

/* Prepare the CPC */
"KEYBOARDSTROKE WAIT SYMBOL AFTER 256"
"KEYBOARDSTROKE WAIT MEMORY &7FFF"
SAY "Done." LF

/* Pause ACE while working */
"PAUSE"

/* Compute tables */
SAY "Computing & poking tables into ACE..."
a=X2D(9000)
b=-0.2
n=52
x=0
DO ang=0 TO 359 BY 2
    n=n+b
    valeur=TRUNC((SIN(DEG(ang))*COS(DEG(ang))+1)*n)
    "POKE ADR" a "VAL" valeur
    IF ang=180 THEN b=-b
    a=a+1
    x=x+1
END
SAY "1st table done:" x "values computed!"

a=X2D(9800)
x=0
DO ang=-90 TO 629 BY 6
    valeur=TRUNC((SIN(DEG(ang))+1)*23+46)
    "POKE ADR" a "VAL" valeur
    a=a+1
    x=x+1
END
DO ang=90 TO 809 BY 6
    valeur=TRUNC((SIN(DEG(ang))+1)*23)
    "POKE ADR" a "VAL" valeur
    a=a+1
    x=x+1
END
SAY "2nd table done:" x "values computed!"
SAY "Done." LF

/* Assemble code */
SAY "Assembling source code..."
ADDRESS "COMMAND" "vasmz80_oldstyle -Fbin -nocase -quiet -L T:CrossDevExample.lst -o T:CrossDevExample.bin CrossDevExample.asm"

/* Check for assembling error... */
IF RC~=0 THEN DO
    "START"
    EXIT
END
SAY "Done." LF

/* Load code into ACE! */
SAY "Loading code into ACE..."
"LOAD FILE T:CrossDevExample.bin ADR 0x8000"
SAY "Done." LF

/* Start ACE again */
"START"

/* Auto-launch the assembled program */
SAY "Auto-launching the assembled program!"
"KEYBOARDSTROKE CALL &8000"
SAY "Done." LF

EXIT

/*
 * Version checker (ver.rev[beta])
 */
CheckVersion:
    PARSE ARG cv "." cr "beta" , rv "." rr "beta"

    IF cv<rv | (cv=rv & cr<rr) THEN
        RETURN 5
    RETURN 0

/*
 * External lib loader
 */
LoadLib:
    PARSE ARG lib

    IF SHOW("L",lib)=0 THEN
        IF ADDLIB(lib,0,-30,0)=0 THEN
            CALL FatalError(lib "not found!*NPlease install it.","Okay!")
    RETURN 0

/*
 * Fatal error
 */
FatalError:
    PARSE ARG message,button

    ADDRESS "COMMAND" 'RequestChoice >NIL: TITLE "'||appname||'" BODY "'||message||'" GADGETS "'||button||'"'
    RETURN 20
